"use strict";

var _interopRequire = function (obj) { return obj && obj.__esModule ? obj["default"] : obj; };

var Declaration = _interopRequire(require("./declaration"));

var Processor = _interopRequire(require("./processor"));

var Comment = _interopRequire(require("./comment"));

var AtRule = _interopRequire(require("./at-rule"));

var vendor = _interopRequire(require("./vendor"));

var parse = _interopRequire(require("./parse"));

var list = _interopRequire(require("./list"));

var Rule = _interopRequire(require("./rule"));

var Root = _interopRequire(require("./root"));

var postcss = function postcss() {
    for (var _len = arguments.length, plugins = Array(_len), _key = 0; _key < _len; _key++) {
        plugins[_key] = arguments[_key];
    }

    if (plugins.length === 1 && Array.isArray(plugins[0])) {
        plugins = plugins[0];
    }
    return new Processor(plugins);
};

postcss.plugin = function (name, initializer) {
    var creator = function creator() {
        var transformer = initializer.apply(this, arguments);
        transformer.postcssPlugin = name;
        transformer.postcssVersion = Processor.prototype.version;
        return transformer;
    };

    creator.postcss = creator();
    return creator;
};

postcss.vendor = vendor;

postcss.parse = parse;

postcss.list = list;

postcss.comment = function (defaults) {
    return new Comment(defaults);
};
postcss.atRule = function (defaults) {
    return new AtRule(defaults);
};
postcss.decl = function (defaults) {
    return new Declaration(defaults);
};
postcss.rule = function (defaults) {
    return new Rule(defaults);
};
postcss.root = function (defaults) {
    return new Root(defaults);
};

module.exports = postcss;